var matchdep = require('matchdep');


module.exports = function(grunt) {

    // load all grunt plugins from node_modules folder
    matchdep.filterAll('grunt-*').forEach(grunt.loadNpmTasks);

    var setVersion = function(src) {

        var npm_package = grunt.file.readJSON('package.json');
        var result = src.replace('{{VERSION}}', npm_package.version);

        return result;

    };

    var concat = {
      qunit:{
        src:['lib/error-formatter.js', 'dist/nucleo-amd.browser.js', 'tests/integration/main.js', 'tests/**/*.test.js'],
        dest:'tests/setup/qunit/tests.js'
      },
      browser: {
        options: {
            banner: '(function(mainContext) {\n',
            footer: '\n mainContext.nucleoAMD = NMD;})(this);',
            process: setVersion
        },
        src: ['lib/core.js', 'lib/error-formatter.js', 'lib/exposed.js', 'lib/plugins.js'],
        dest:'dist/nucleo-amd.browser.js'
      },
      nodejs: {
        options: {
            footer: '\n module.exports = NMD;',
            process: setVersion
        },
        src: ['lib/core.js', 'lib/error-formatter.js', 'lib/exposed.js', 'lib/plugins.js'],
        dest:'dist/nucleo-amd.nodejs.js'
      },
      binaries: {
        files:{
          'bin/nucleo-amd.nodejs.js': ['dist/nucleo-amd.nodejs.js'],
          'bin/nucleo-amd.browser.js':['dist/nucleo-amd.browser.js']
        }
      }
    };



  grunt.initConfig({
    pkg: grunt.file.readJSON('package.json'),
    concat: concat,
    uglify: {
        options: {
            mangle:     true,
            compress:   true,
            banner:     '/* <%= pkg.name %> v<%= pkg.version %>  */'
        },
        prod:    {
            files: {
                'bin/nucleo-amd.min.js': ['bin/nucleo-amd.browser.js']
            }
        }
    },
    express: {
        dev: {
            options: {
                script: 'tests/setup/server/server.js'
            }
        }
    },
    shell: {
        pre_commit_tests: {
          command: 'testem ci'
        }
    },
    watch: {
        options: {
            livereload: true,
            nospawn:    true
        },
        concat: {
            files: [
                'lib/**/*.js',
                'tests/integration/**/*.js'
            ],
            tasks: ['concat:browser', 'concat:qunit']
        },
    },
    bump: {
      options: {
        files: ['package.json', 'bower.json'],
        updateConfigs: ['pkg'],
        commit: true,
        commitMessage: 'Release v%VERSION%',
        commitFiles: ['-a'],
        createTag: true,
        tagName: '%VERSION%',
        tagMessage: '%VERSION%',
        push: true,
        pushTo: 'origin',
        gitDescribeOptions: '--tags --always --abbrev=1 --dirty=-d',
        globalReplace: true,
        prereleaseName: false,
        regExp: false
      }
    },
    release: {
        options: {
            bump: false,
            commit: false,
            tag: false,
            pushTags: false,
            npm: true
        }
    }

  });

// TODOS:
// - include a task to automaticaly create changelogs based on commits - https://github.com/lalitkapoor/github-changes

  grunt.registerTask('testing', ['concat:browser', 'concat:qunit', 'express', 'watch']);

  grunt.registerTask('build:lib', ['concat:nodejs', 'concat:browser']);
  grunt.registerTask('build:prod', ['build:lib', 'concat:binaries', 'uglify']);

  grunt.registerTask('deploy:patch', ['bump-only:patch', 'build:prod', 'shell:pre_commit_tests', 'bump-commit', 'release:patch']);
  grunt.registerTask('deploy:minor', ['bump-only:minor', 'build:prod', 'shell:pre_commit_tests', 'bump-commit', 'release:minor']);
  grunt.registerTask('deploy:major', ['bump-only:major', 'build:prod', 'shell:pre_commit_tests', 'bump-commit', 'release:major']);
  grunt.registerTask('deploy',       ['deploy:patch']);

  grunt.registerTask('default', ['build:prod']);

};
